<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>{% block title %}Annal{% endblock %}</title>
  <link rel="stylesheet" href="/static/style.css">
  <script src="https://unpkg.com/htmx.org@2.0.4"></script>
  <script src="https://unpkg.com/htmx-ext-sse@2.2.2/sse.js"></script>
</head>
<body>
  <nav class="topnav">
    <a href="/" class="nav-brand">
      <span class="brand-icon">&gt;_</span>
      <span class="brand-text">annal</span>
    </a>
    <div class="nav-links">
      <a href="/">projects</a>
    </div>
    <div class="activity-indicator" id="activity-indicator"></div>
  </nav>
  <main class="container">
    {% block content %}{% endblock %}
  </main>
</body>
</html>
